/*
 * Copyright 2009 Ilya Epifanov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.xajile.fuchakubutsu;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class FrozenObjectPart implements Part {
  private final List<Object> objects;

  public FrozenObjectPart(List<Object> objects) {
    this.objects = Collections.unmodifiableList(objects);
  }

  public FrozenObjectPart(Object... objects) {
    this.objects = Collections.unmodifiableList(Arrays.asList(objects));
  }

  public List<Object> getObjects() {
    return objects;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;

    FrozenObjectPart part = (FrozenObjectPart) o;

    return !(getObjects() != null ? !getObjects().equals(part.getObjects()) : part
            .getObjects() != null);
  }

  @Override
  public int hashCode() {
    return objects != null ? objects.hashCode() : 0;
  }
}
