/*
 * Copyright 2009 Ilya Epifanov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.xajile.fuchakubutsu;

abstract class ParserActions {
  private int lastObjectBeginning;
  private int lastConsumed;
  protected char[] data;
  private final EventHandler handler;

  protected ParserActions(EventHandler handler) {
    this.handler = handler;
  }

  protected final void setData(char[] data) {
    this.data = data;
  }

  protected final void beginObject(int p) {
    lastObjectBeginning = p;
  }

  protected final void endObject(int p) {
    int lparen = lastObjectBeginning - 3;
    if (lparen > lastConsumed) {
      consumeText(lparen);
      lastConsumed = lparen;
    }
    consumeObject(p);
    lastConsumed = p;
  }

  private void consumeObject(int p) {
    handler.handleObject(data, lastConsumed + 3, p - lastConsumed - 6);
  }

  private void consumeText(int p) {
    handler.handleText(data, lastConsumed, p - lastConsumed);
  }

  protected final void eof(int p) {
    if (p > lastConsumed) {
      consumeText(p);
    }
    handler.finish();
  }
}
