/*
 * Copyright 2009 Ilya Epifanov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.xajile.fuchakubutsu;

import java.io.IOException;
import java.io.Writer;

public class TextPart implements Part {
  private final char[] buf;
  private final int offset;
  private final int length;

  public TextPart(String str) {
    this(str.toCharArray());
  }

  public TextPart(char[] buf) {
    this(buf, 0, buf.length);
  }

  public TextPart(char[] buf, int offset, int length) {
    this.buf = buf;
    this.offset = offset;
    this.length = length;
  }

  public String toString() {
    StringBuilder ret = new StringBuilder();
    ret.append("t:").append(buf, offset, length);
    return ret.toString();
  }

  public char[] getBuf() {
    return buf;
  }

  public int getOffset() {
    return offset;
  }

  public int getLength() {
    return length;
  }

  public void write(Writer out) throws IOException {
    out.write(buf, offset, length);
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;

    TextPart textPart = (TextPart) o;

    if (length != textPart.length) return false;

    for (int o1 = offset, o2 = textPart.offset, i = 0; i < length; i++, o1++, o2++) {
      if (buf[o1] != textPart.buf[o2])
        return false;
    }
    return true;
  }

  @Override
  public int hashCode() {
    int result = 19;
    for (int i = offset; i < offset + length; i++) {
      result = 31 * result + buf[i];
    }
    result = 31 * result + length;
    return result;
  }
}
