/*
 * Copyright 2009 Ilya Epifanov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ru.xajile.fuchakubutsu.tsuyujimo;

import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;

abstract class ParserActions {
  private boolean negativeNumeric;
  private long numeric;
  protected char[] data;
  protected int off;
  protected int len;
  private char c1, c2, c3, c4;
  private int writeShift;
  private int stringP;
  private final EventHandler handler;
  private short dateYear, dateMonth, dateDay, dateHour, dateMinutes, dateSeconds;

  protected ParserActions(EventHandler handler) {
    this.handler = handler;
  }

  protected final void setData(char[] data, int off, int len) {
    this.data = data;
    this.off = off;
    this.len = len;
  }

  protected final void negativeNumeric() {
    negativeNumeric = true;
  }

  protected final void initNumeric() {
    negativeNumeric = false;
    numeric = 0;
  }

  protected final void addUpNumeric(int n) {
    numeric = numeric * 10 + n;
  }

  protected final void finishInt() {
    handler.handleInt((int) (negativeNumeric ? -numeric : numeric));
  }

  protected final void finishLong() {
    handler.handleLong(negativeNumeric ? -numeric : numeric);
  }

  protected final void initString(int p) {
    writeShift = 0;
    stringP = p;
  }

  protected final void h1(char c) {
    c1 = (char) (c - '0');
  }

  protected final void h1h(char c) {
    c1 = (char) (c - 'a' + 10);
  }

  protected final void h2(char c) {
    c2 = (char) (c - '0');
  }

  protected final void h2h(char c) {
    c2 = (char) (c - 'a' + 10);
  }

  protected final void h3(char c) {
    c3 = (char) (c - '0');
  }

  protected final void h3h(char c) {
    c3 = (char) (c - 'a' + 10);
  }

  protected final void h4(char c) {
    c4 = (char) (c - '0');
  }

  protected final void h4h(char c) {
    c4 = (char) (c - 'a' + 10);
  }

  protected final void finishCharInString(int p) {
    data[p + writeShift] = data[p];
  }

  protected final void finishEncodedCharInString(int p) {
    writeShift -= 4;
    data[p + writeShift - 1] = (char) ((c1 << 12) | (c2 << 8) | (c3 << 4) | c4);
  }

  protected final void finishString(int p) {
    handler.handleString(data, stringP, (p + writeShift - stringP));
  }

  protected final void finishChar(char c) {
    handler.handleChar(c);
  }

  protected final void finishEncodedChar() {
    handler.handleChar((char) ((c1 << 12) | (c2 << 8) | (c3 << 4) | c4));
  }

  protected final void finishBoolTrue() {
    handler.handleBool(true);
  }

  protected final void finishBoolFalse() {
    handler.handleBool(false);
  }

  protected final void finishNull() {
    handler.handleNull();
  }

  protected final void dateYear() {
    this.dateYear = (short) numeric;
  }

  protected final void dateMonth() {
    this.dateMonth = (short) numeric;
  }

  protected final void dateDay() {
    this.dateDay = (short) numeric;
  }

  protected final void dateHour() {
    this.dateHour = (short) numeric;
  }

  protected final void dateMinutes() {
    this.dateMinutes = (short) numeric;
  }

  protected final void dateSeconds() {
    this.dateSeconds = (short) numeric;
  }

  protected final void finishDateWithMillis() {
    handler.handleDate(
            new DateTime(dateYear, dateMonth, dateDay, dateHour, dateMinutes, dateSeconds,
                    (int) numeric, DateTimeZone.UTC));
  }
}
